UOrd

Struct UOrd 

pub struct UOrd<T, const N: usize> { /* private fields */ }
An unordered tuple of items of type T and length N.

UOrd is implemented such that the order of elements on creation does not matter, and the UOrds created from different ordered lists will behave similarly in PartialEq, Ord, or Hash.

UOrd’s implementation maintains a sorted list of values that is not allowed to be mutated. Using interior mutability to mutate elements of a UOrd is a logic error and is not supported. Doing so is very likely to cause strange behavior.

Implementations

Source

impl<T, const N: usize> UOrd<T, N>
where T: Ord,

pub fn new(values: [T; N]) -> Self

Create a new UOrd from an array of values.

pub fn contains<Q>(&self, x: &Q) -> bool
where T: Borrow<Q>, Q: Eq + ?Sized,

Tests whether this UOrd contains the given value.

pub fn replace<Q>(&self, from: &Q, to: &T) -> Self
where T: Borrow<Q> + Clone, Q: Eq + ?Sized,

Replaces any occurrence of from with to, creating a new UOrd.

pub fn map<U>(self, f: impl FnMut(T) -> U) -> UOrd<U, N>
where U: Ord,

Applies the function f to each element of this UOrd, returning a new UOrd.

pub fn map_each_ref<U>(&self, f: impl FnMut(&T) -> U) -> UOrd<U, N>
where U: Ord,

Applies the function f to a reference to each element of this UOrd, returning a new UOrd.

pub fn try_map_opt<U>(self, f: impl FnMut(T) -> Option<U>) -> Option<UOrd<U, N>>
where U: Ord,

Available on crate feature alloc only.

Applies a fallible function f to each element of this UOrd, returning a value of type Option<UOrd<U, N>>. The provided function takes a T and must return a value of type Option<U>.

This function is temporary, and will be deprecated when try_trait_v2 is stabilized.

pub fn try_map_res<U, E>( self, f: impl FnMut(T) -> Result<U, E>, ) -> Result<UOrd<U, N>, E>
where U: Ord,

Available on crate feature alloc only.

Applies a fallible function f to each element of this UOrd, returning a value of type Result<UOrd<U, N>, E>. The provided function takes a T and must return a value of type Return<U, E>.

This function is temporary, and will be deprecated when try_trait_v2 is stabilized.

pub fn convert<U>(self) -> UOrd<U, N>
where T: Into<U>, U: Ord,

Converts each element of this UOrd into a given type U based on T’s Into<U> implementation.

pub fn try_convert<U>(self) -> Result<UOrd<U, N>, T::Error>
where T: TryInto<U>, U: Ord,

Available on crate feature alloc only.

Fallibly converts each element of this UOrd into a given type U based on T’s TryInto<U> implementation, alternatively returning error.

pub fn test_all(&self, f: impl FnMut(&T) -> bool) -> bool

Tests if all elements pass the given predicate.

Equivalent to self.iter().all(f).

pub fn test_any(&self, f: impl FnMut(&T) -> bool) -> bool

Tests if any element passes the given predicate.

Equivalent to self.iter().any(f).

pub fn make_proxied<P: Proxy<T>>(self) -> UOrdProxied<T, N, P>

Converts this UOrd into a UOrdProxied by wrapping its contained elements of type T in ProxyWrappers, using the provided Proxy P for comparison and equality.

Source

impl<T, const N: usize> UOrd<T, N>

pub const fn min(&self) -> &T

Gets the first (smallest) element in the internal list.

Panics

This function will panic if N == 0.

pub const fn max(&self) -> &T

Gets the last (greatest) element in the internal list.

Panics

This function will panic if N == 0.

pub fn each_ref(&self) -> UOrd<&T, N>

Borrows each element in this UOrd, returning a UOrd of those references.

pub fn as_ref_array(&self) -> [&T; N]

Borrows each element in this UOrd, returning an array of those references.

pub const fn as_array(&self) -> &[T; N]

Returns an immutable reference to the array of elements, which is sorted.

pub const fn into_array(self) -> [T; N]

Converts this UOrd into its array of elements, which is sorted.

pub fn into_tuple(self) -> <[T; N] as IntoTuple>::Tuple
where [T; N]: IntoTuple,

Converts this UOrd into a tuple of length N similarly to into_array.

pub fn iter(&self) -> UOrdIter<'_, T, N>

Creates an iterator over references to each element of this UOrd.

Source

impl<T, const N: usize, P> UOrd<ProxyWrapper<T, P>, N>
where P: Proxy<T>,

pub fn new_proxied(values: [T; N]) -> Self

Create a new UOrdProxied from an array of values, utilizing a custom Proxy for comparison and equality.

pub fn contains_proxied<Q>(&self, x: &Q) -> bool
where T: Borrow<Q>, Q: ?Sized, for<'q> P: Proxy<&'q Q>,

Tests whether this UOrdProxied contains the given value.

pub fn replace_proxied<Q>(&self, from: &Q, to: &T) -> Self
where T: Borrow<Q> + Clone, Q: ?Sized, for<'q> P: Proxy<&'q Q>,

Replaces any occurrence of from with to, creating a new UOrdProxied.

pub const fn min_proxied(&self) -> &T

Gets the first (smallest) element in the internal list, stripped of its wrappers.

Panics

This function will panic if N == 0.

pub const fn max_proxied(&self) -> &T

Gets the last (greatest) element in the internal list, stripped of its wrappers.

Panics

This function will panic if N == 0.

pub fn each_ref_proxied(&self) -> UOrd<&T, N>
where T: Ord,

Borrows each element in this UOrdProxied, returning a UOrd of those references, stripped of their wrappers.

pub fn as_ref_array_proxied(&self) -> [&T; N]

Borrows each element in this UOrdProxied, returning an array of those references, stripped of their wrappers.

pub const fn as_array_proxied(&self) -> &[T; N]

Returns an immutable reference to the array of elements, which is sorted, and has each element stripped of their wrappers.

pub const fn into_array_proxied(self) -> [T; N]

Converts this UOrdProxied into its array of elements, which is sorted, and has each element stripped of their wrappers.

pub fn into_tuple_proxied(self) -> <[T; N] as IntoTuple>::Tuple
where [T; N]: IntoTuple,

Converts this UOrd into a tuple of length N similarly to into_array, and has each element stripped of their wrappers..

pub fn map_proxied<U, R>(self, f: impl FnMut(T) -> U) -> UOrdProxied<U, N, R>
where R: Proxy<U>,

Applies the function f to each element of this UOrdProxied, returning a new UOrdProxied. This function also allows a proxy, R, for the new UOrdProxied to be specified.

pub fn map_each_ref_proxied<U, R>( &self, f: impl FnMut(&T) -> U, ) -> UOrdProxied<U, N, R>
where R: Proxy<U>,

Applies the function f to a reference to each element of this UOrdProxied, returning a new UOrdProxied. This function also allows a proxy, R, for the new UOrdProxied to be specified.

pub fn try_map_proxied_opt<U, R>( self, f: impl FnMut(T) -> Option<U>, ) -> Option<UOrdProxied<U, N, R>>
where R: Proxy<U>,

Available on crate feature alloc only.

Applies a fallible function f to each element of this UOrdProxied, returning a value of type Option<UOrdProxied<U, N, R>>. The provided function takes a T and must return a value of type Option<U>. This function also allows a proxy, R, for the new UOrdProxied to be specified.

This function is temporary, and will be deprecated when try_trait_v2 is stabilized.

pub fn try_map_proxied_res<U, R, E>( self, f: impl FnMut(T) -> Result<U, E>, ) -> Result<UOrdProxied<U, N, R>, E>
where R: Proxy<U>,

Available on crate feature alloc only.

Applies a fallible function f to each element of this UOrdProxied, returning a value of type Result<UOrdProxied<U, N, R>, E>. The provided function takes a T and must return a value of type Return<U, E>. This function also allows a proxy, R, for the new UOrdProxied to be specified.

This function is temporary, and will be deprecated when try_trait_v2 is stabilized.

pub fn convert_proxied<U, R>(self) -> UOrdProxied<U, N, R>
where T: Into<U>, R: Proxy<U>,

Converts each element of this UOrdProxied into a given type U based on T’s Into<U> implementation.

pub fn try_convert_proxied<U, R>(self) -> Result<UOrdProxied<U, N, R>, T::Error>
where T: TryInto<U>, R: Proxy<U>,

Available on crate feature alloc only.

Fallibly converts each element of this UOrdProxied into a given type U based on T’s TryInto<U> implementation, alternatively returning error.

pub fn test_all_proxied(&self, f: impl FnMut(&T) -> bool) -> bool

Tests if all elements pass the given predicate.

Equivalent to self.iter_proxied().all(f).

pub fn test_any_proxied(&self, f: impl FnMut(&T) -> bool) -> bool

Tests if any element passes the given predicate.

Equivalent to self.iter_proxied().any(f).

pub fn make_unproxied(self) -> UOrd<T, N>
where T: Ord,

Converts this UOrdProxied into a UOrd, unwrapping the contained ProxyWrappers, using the default implementation of Ord on T for comparison and equality.

pub fn iter_proxied(&self) -> UOrdProxiedIter<'_, T, N, P>

Creates an iterator over references to each element of this UOrdProxied, stripped of their wrappers.

pub fn into_iter_proxied(self) -> UOrdProxiedIntoIter<T, N, P>

Creates an iterator over each element of this UOrdProxied, stripped of their wrappers.

Source

impl<T> UOrd<T, 2>
where T: Ord,

pub fn is_distinct(&self) -> bool

Returns true if this pair was distinct (the values in it were not equal to each other) otherwise returns false.

pub fn other<Q>(&self, x: &Q) -> Option<&T>
where T: Borrow<Q>, Q: Eq + ?Sized,

If this unordered pair contains the given value, this function will return the other value. If this unordered pair did not contain the given value, this will return None.

Note that this function does not care about the distinctness of the pair, and will still return the other value, even if it was equal to the input value. If you need this behavior, see UOrd::other_distinct.

pub fn other_distinct<Q>(&self, x: &Q) -> Option<&T>
where T: Borrow<Q>, Q: Eq + ?Sized,

If this unordered pair contains the given value, this function will return the other value. If this unordered pair did not contain the given value, this will return None.

Additionally, this function will return None if the two items in this pair were equal, guaranteeing that the output value is never equal to the input value in the case of an indistinct pair.

Source

impl<T, P> UOrd<ProxyWrapper<T, P>, 2>
where P: Proxy<T>,

pub fn other_proxied<Q>(&self, x: &Q) -> Option<&T>
where T: Borrow<Q>, Q: ?Sized, for<'q> P: Proxy<&'q Q>,

If this unordered pair contains the given value, this function will return the other value. If this unordered pair did not contain the given value, this will return None.

Note that this function does not care about the distinctness of the pair, and will still return the other value, even if it was equal to the input value. If you need this behavior, see UOrdProxied::other_distinct_proxied.

pub fn other_distinct_proxied<Q>(&self, x: &Q) -> Option<&T>
where T: Borrow<Q>, Q: ?Sized, for<'q> P: Proxy<&'q Q>,

If this unordered pair contains the given value, this function will return the other value. If this unordered pair did not contain the given value, this will return None.

Additionally, this function will return None if the two items in this pair were equal, guaranteeing that the output value is never equal to the input value in the case of an indistinct pair.